@@ -68,8 +68,12 @@ struct _GVfsUDisks2VolumeMonitor
 
   GSettings *lockdown_settings;
   gboolean readonly_lockdown;
+
+  gint update_id;
 };
 
+#define UPDATE_TIMEOUT 100 /* ms */
+
 static UDisksClient *get_udisks_client_sync (GError **error);
 
 static void update_all               (GVfsUDisks2VolumeMonitor  *monitor,
@@ -148,6 +152,8 @@ gvfs_udisks2_volume_monitor_finalize (GObject *object)
 
   g_clear_object (&monitor->lockdown_settings);
 
+  g_clear_handle_id (&monitor->update_id, g_source_remove);
+
   G_OBJECT_CLASS (gvfs_udisks2_volume_monitor_parent_class)->finalize (object);
 }
 
@@ -425,12 +431,33 @@ gvfs_udisks2_volume_monitor_get_readonly_lockdown (GVfsUDisks2VolumeMonitor *mon
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static gboolean
+update_func (gpointer user_data)
+{
+  GVfsUDisks2VolumeMonitor *monitor = GVFS_UDISKS2_VOLUME_MONITOR (user_data);
+
+  monitor->update_id = 0;
+
+  update_all (monitor, TRUE, FALSE);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+schedule_update (GVfsUDisks2VolumeMonitor *monitor)
+{
+  if (monitor->update_id != 0)
+    return;
+
+  monitor->update_id = g_timeout_add (UPDATE_TIMEOUT, update_func, monitor);
+}
+
 void
 gvfs_udisks2_volume_monitor_update (GVfsUDisks2VolumeMonitor *monitor)
 {
   g_return_if_fail (GVFS_IS_UDISKS2_VOLUME_MONITOR (monitor));
   udisks_client_settle (monitor->client);
-  update_all (monitor, TRUE, FALSE);
+  schedule_update (monitor);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -532,7 +559,7 @@ on_client_changed (UDisksClient  *client,
                    gpointer       user_data)
 {
   GVfsUDisks2VolumeMonitor *monitor = GVFS_UDISKS2_VOLUME_MONITOR (user_data);
-  update_all (monitor, TRUE, FALSE);
+  schedule_update (monitor);
 }
 
 static void
@@ -540,7 +567,7 @@ mountpoints_changed (GUnixMountMonitor *mount_monitor,
                      gpointer           user_data)
 {
   GVfsUDisks2VolumeMonitor *monitor = GVFS_UDISKS2_VOLUME_MONITOR (user_data);
-  update_all (monitor, TRUE, FALSE);
+  schedule_update (monitor);
 }
 
 static void
@@ -548,7 +575,7 @@ mounts_changed (GUnixMountMonitor *mount_monitor,
                 gpointer           user_data)
 {
   GVfsUDisks2VolumeMonitor *monitor = GVFS_UDISKS2_VOLUME_MONITOR (user_data);
-  update_all (monitor, TRUE, FALSE);
+  schedule_update (monitor);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -622,7 +649,7 @@ should_include (const gchar *mount_path,
   gboolean ret = FALSE;
   const gchar *home_dir = NULL;
   const gchar *user_name;
-  gsize user_name_len;
+  const gchar *p;
 
   g_return_val_if_fail (mount_path != NULL, FALSE);
 
@@ -650,23 +677,10 @@ should_include (const gchar *mount_path,
   if (g_unix_is_mount_path_system_internal (mount_path))
     goto out;
 
-  /* Only display things in
-   * - /media; and
-   * - $HOME; and
-   * - /run/media/$USER
-   */
-
   /* Hide mounts within a subdirectory starting with a "." - suppose it was a purpose to hide this mount */
   if (g_strstr_len (mount_path, -1, "/.") != NULL)
     goto out;
 
-  /* Check /media */
-  if (g_str_has_prefix (mount_path, "/media/"))
-    {
-      ret = TRUE;
-      goto out;
-    }
-
   /* Check home dir */
   home_dir = g_get_home_dir ();
   if (home_dir != NULL)
@@ -678,15 +692,23 @@ should_include (const gchar *mount_path,
         }
     }
 
-  /* Check /run/media/$USER/ */
-  user_name = g_get_user_name ();
-  user_name_len = strlen (user_name);
-  if (strncmp (mount_path, "/run/media/", sizeof ("/run/media/") - 1) == 0 &&
-      strncmp (mount_path + sizeof ("/run/media/") - 1, user_name, user_name_len) == 0 &&
-      mount_path[sizeof ("/run/media/") - 1 + user_name_len] == '/')
+  /* Display mounts that are direct descendants of /media/ resp. /run/media/,
+   * or mounts with /media/$USER/ resp. /run/media/$USER/ prefix.
+   */
+  p = mount_path;
+  if (g_str_has_prefix (p, "/run/"))
+    p += sizeof ("/run") - 1;
+  if (g_str_has_prefix (p, "/media/"))
     {
-      ret = TRUE;
-      goto out;
+      p += sizeof ("/media/") - 1;
+
+      user_name = g_get_user_name ();
+      if ((g_str_has_prefix (p, user_name) && p[strlen (user_name)] == '/') ||
+          g_strrstr (p, "/") == NULL)
+        {
+          ret = TRUE;
+          goto out;
+        }
     }
 
  out:
